@@ -12,11 +12,13 @@ urlpatterns = patterns('',
     url(r'^$',
         direct_to_template,
         {'template': 'static/promo.html'},
-        name='promo')
+        name='promo'),
+    (r'^i18n/', include('django.conf.urls.i18n')),
 )
 
 if settings.DEBUG:
     urlpatterns += patterns('',
-        (r'^media/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.MEDIA_ROOT, 'show_indexes': True, }),
-        (r'^docs/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.DOCUMENT_ROOT, 'show_indexes': True, }),
+        (r'^media/(?P<path>.*)$',
+         'django.views.static.serve',
+         {'document_root': settings.MEDIA_ROOT, 'show_indexes': True, }),
 )
