@@ -8,6 +8,17 @@ import sys
 os.environ["DJANGO_SETTINGS_MODULE"] = 'demo_project.settings'
 from demo_project import settings
 
+settings.INSTALLED_APPS = (
+    'django.contrib.auth',
+    'django.contrib.sessions',
+    'django.contrib.contenttypes',
+    'django.contrib.admin',
+    'django.contrib.sites',
+    'demo_project.profiles',
+    'guardian',
+    'userena',
+)
+
 def main():
     from django.test.utils import get_runner
     test_runner = get_runner(settings)(interactive=False)
