@@ -1,9 +1,11 @@
 from django import forms
-from django.contrib.auth.models import User
 from django.utils.translation import ugettext_lazy as _
 from django.contrib.auth import authenticate
+from django.contrib.auth.models import User
 
 from userena import settings as userena_settings
+from userena.models import UserenaSignup
+from userena.utils import get_profile_model
 
 attrs_dict = {'class': 'required'}
 
@@ -12,7 +14,8 @@ class SignupForm(forms.Form):
     Form for creating a new user account.
 
     Validates that the requested username and e-mail is not already in use.
-    Also requires the password to be entered twice. A TOS is also required to be seen.
+    Also requires the password to be entered twice and the Terms of Service to
+    be accepted.
 
     """
     username = forms.RegexField(regex=r'^\w+$',
@@ -22,15 +25,13 @@ class SignupForm(forms.Form):
                                 error_messages={'invalid': _('Username must contain only letters, numbers and underscores.')})
     email = forms.EmailField(widget=forms.TextInput(attrs=dict(attrs_dict,
                                                                maxlength=75)),
-                             label=_("Email address"))
-    password1 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict, render_value=False),
+                             label=_("Email"))
+    password1 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict,
+                                                           render_value=False),
                                 label=_("Password"))
-    password2 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict, render_value=False),
-                                label=_("Password (again)"))
-
-    tos = forms.BooleanField(widget=forms.CheckboxInput(attrs=attrs_dict),
-                             label=_(u'I have read and agree to the Terms of Service'),
-                             error_messages={'required': _('You must agree to the terms to register.')})
+    password2 = forms.CharField(widget=forms.PasswordInput(attrs=attrs_dict,
+                                                           render_value=False),
+                                label=_("Confirm password"))
 
     def clean_username(self):
         """
@@ -50,17 +51,16 @@ class SignupForm(forms.Form):
         return self.cleaned_data['username']
 
     def clean_email(self):
-        """ Validate that the e-mail address is unique """
+        """ Validate that the e-mail address is unique. """
         if User.objects.filter(email__iexact=self.cleaned_data['email']):
-            raise forms.ValidationError(_('This email address is already in use. Please supply a different email address.'))
+            raise forms.ValidationError(_('This email is already in use. Please supply a different email.'))
         return self.cleaned_data['email']
 
     def clean(self):
         """
-        Verifiy that the values entered into the two password fields
-        match. Note that an error here will end up in
-        ``non_field_errors()`` because it doesn't apply to a single
-        field.
+        Validates that the values entered into the two password fields match.
+        Note that an error here will end up in ``non_field_errors()`` because
+        it doesn't apply to a single field.
 
         """
         if 'password1' in self.cleaned_data and 'password2' in self.cleaned_data:
@@ -68,6 +68,21 @@ class SignupForm(forms.Form):
                 raise forms.ValidationError(_('The two password fields didn\'t match.'))
         return self.cleaned_data
 
+    def save(self):
+        """ Creates a new user and account. Returns the newly created user. """
+        username, email, password = (self.cleaned_data['username'],
+                                     self.cleaned_data['email'],
+                                     self.cleaned_data['password1'])
+
+        new_user = UserenaSignup.objects.create_inactive_user(username, email, password)
+        return new_user
+
+class SignupFormTos(SignupForm):
+    """ Add a Terms of Service button to the ``SignupForm``. """
+    tos = forms.BooleanField(widget=forms.CheckboxInput(attrs=attrs_dict),
+                             label=_(u'I have read and agree to the Terms of Service'),
+                             error_messages={'required': _('You must agree to the terms to register.')})
+
 class AuthenticationForm(forms.Form):
     """
     A custom ``AuthenticationForm`` where the identification can be a e-mail
@@ -75,14 +90,22 @@ class AuthenticationForm(forms.Form):
 
     """
     identification = forms.CharField(label=_("Email or username"),
+                                     widget=forms.TextInput(attrs=attrs_dict),
                                      max_length=75,
-                                     error_messages={'required': _('Either supply us with your e-mail address or username.')})
-    password = forms.CharField(label=_("Password"), widget=forms.PasswordInput)
+                                     error_messages={'required': _('Either supply us with your email or username.')})
+    password = forms.CharField(label=_("Password"),
+                               widget=forms.PasswordInput(attrs=attrs_dict, render_value=False))
     remember_me = forms.BooleanField(widget=forms.CheckboxInput(attrs=attrs_dict),
                                      required=False,
                                      label=_(u'Remember me for %(days)s' % {'days': userena_settings.USERENA_REMEMBER_ME_DAYS[0]}))
 
     def clean(self):
+        """
+        Checks for the identification and password.
+
+        If the combination can't be found will raise an invalid signin error.
+
+        """
         identification = self.cleaned_data.get('identification')
         password = self.cleaned_data.get('password')
 
@@ -95,12 +118,14 @@ class AuthenticationForm(forms.Form):
 class ChangeEmailForm(forms.Form):
     email = forms.EmailField(widget=forms.TextInput(attrs=dict(attrs_dict,
                                                                maxlength=75)),
-                             label=_("Email address"))
+                             label=_("New email"))
 
     def __init__(self, user, *args, **kwargs):
         """
-        The current ``user`` needs to be supplied with this form so that we can
-        check if the e-mail address is still free.
+        The current ``user`` is needed for initialisation of this form so
+        that we can check if the email address is still free and not always
+        returning ``True`` for this query because it's the users own e-mail
+        address.
 
         """
         super(ChangeEmailForm, self).__init__(*args, **kwargs)
@@ -108,13 +133,53 @@ class ChangeEmailForm(forms.Form):
             raise TypeError, "user must be an instance of User"
         else: self.user = user
 
+        self.fields['email'].help_text = _('Your current email is %(email)s' % \
+                                           {'email': user.email})
+
     def clean_email(self):
-        """ Validate that the e-mail address is not already registered with another user """
+        """ Validate that the email is not already registered with another user """
         if self.cleaned_data['email'].lower() == self.user.email:
-            raise forms.ValidationError(_('Your already known under this email address.'))
+            raise forms.ValidationError(_('You\'re already known under this email.'))
         if User.objects.filter(email__iexact=self.cleaned_data['email']).exclude(email__iexact=self.user.email):
-            raise forms.ValidationError(_('This email address is already in use. Please supply a different email address.'))
+            raise forms.ValidationError(_('This email is already in use. Please supply a different email.'))
         return self.cleaned_data['email']
 
-class AccountForm(forms.ModelForm):
-    pass
+    def save(self):
+        """
+        Save method calls ``user.change_email()`` method which sends out an
+        email with an verification key to verify and with it enable this new
+        email address.
+
+        """
+        return self.user.userena_signup.change_email(self.cleaned_data['email'])
+
+class EditProfileForm(forms.ModelForm):
+    """ Base form used for fields that are always required """
+    first_name = forms.CharField(label=_('First name'),
+                                 max_length=30,
+                                 required=False)
+    last_name = forms.CharField(label=_('Last name'),
+                                max_length=30,
+                                required=False)
+
+    def __init__(self, *args, **kw):
+        super(forms.ModelForm, self).__init__(*args, **kw)
+        # Put the first and last name at the top
+        new_order = self.fields.keyOrder[:-2]
+        new_order.insert(0, 'first_name')
+        new_order.insert(1, 'last_name')
+        self.fields.keyOrder = new_order
+
+    class Meta:
+        model = get_profile_model()
+        exclude = ['user']
+
+    def save(self, force_insert=False, force_update=False, commit=True):
+        profile = super(EditProfileForm, self).save(commit=commit)
+        # Save first and last name
+        user = profile.user
+        user.first_name = self.cleaned_data['first_name']
+        user.last_name = self.cleaned_data['last_name']
+        user.save()
+
+        return profile
